function [w,CPU_TIME]  = FNLDA(Data, Class)
%--------------------------------------------------------------------------
% FNLDA Fast Null Linear Discriminant Analysis
%
%   [w,CPU_TIME]  = FNLDA(Data, Class)
%
%   Input:                                                             
%     Data:  data matrix (Each row is a data point) (NVec x Dim)
%     Class: class label (class 1, ..., k) 
%
%            Note: All vectors of a class is to be arranged together, e.g.,
%            if there are 2 classes with v1, v2, v3 vectors in class 1 and
%            u1, u2, u3, u4 vectors in class 2 then input Data is to be
%            formed as
%
%            Data = [v1; v2; v3; u1; u2; u3; u4],             (1)
%
%            where the size of vector v or vector u is 1xDim. Therefore the 
%            size of input Data in equation 1 would be 7xDim (depicting 
%            first 3 vectors belong to class 1 and second 4 vectors belong
%            to class 2). In the case of equation 1, class label is to be 
%            given as
%
%            Class = [1;1;1;2;2;2;2].                         (2)
%
%            size of Class (in equation 2) is 7x1.
%         
%   Output:                                                            
%          w: transformation matrix                                  
%    CPU_TIME : computational time                                 
%
%   Reference:
%   Sharma, A. and Paliwal, K.K., A new perspective to null linear
%   discriminant analysis method and its fast implementation using random
%   matrix multiplication with scatter matrices, Pattern Recognition, 2011
%   (in press), DOI: 10.1016/j.patcog.2011.11.018.
%
% 
%   Alok Sharma
%   University of Tokyo
%   Email: aloks@ims.u-tokyo.ac.jp
%
%--------------------------------------------------------------------------
 
t0 = cputime;

k        = max(Class); % number of classes
[Nvec,n] = size(Data);
cc    = sum(Data)/Nvec;

for i = 1:k 
	loc = find(Class==i);
	[num,col] = size(loc);
    vec(i)=num;
end

Ht = (Data - ones(Nvec,1)*cc)';

[E1,Dt] = eig(Ht'*Ht);
rt = rank(Dt);
dt = diag(Dt);
[dt,inx]=sort(dt,'descend');
dt=dt(1:rt);
E1=E1(:,inx(1:rt));
dt1=sqrt(dt);
dt2=sqrt(1./dt);
dt3=1./dt;

Dt=diag(dt1);
Hth=Dt*E1';

% Compute matrix B\hat (equation 10)
for j=1:k
    rg=sum(vec(1:j-1));
    Hbh(:,j)=(1/sqrt(vec(j)))*(sum(Hth(:,1+rg:vec(j)+rg)')');
end
rb = k-1;

% Compute W\hat
rand('state',0);
w = rand(rt,rb);
K = w'*Hbh;
Dt=diag(dt3);
w = Dt*Hbh*K';

% Orthonormalize
[w,T]=qr(w,0);

% Compute orientation matrix
Dt=diag(dt2);
w = Dt*w;
w = E1*w;
w = Ht*w;

CPU_TIME = cputime - t0;
end