function [w,CPU_TIME]  = IPILDA(Data, Class)
%--------------------------------------------------------------------------
% IPILDA: Improved Pseudoinverse Linear Discriminant Analysis
%
%   [w,CPU_TIME]  = IPILDA(Data, Class)
%
%   Input:                                                             
%     Data:  data matrix (Each row is a data point) (NVec x Dim)
%     Class: class label (class 1, ..., k) 
%
%            Note: All vectors of a class is to be arranged together, e.g.,
%            if there are 2 classes with v1, v2, v3 vectors in class 1 and
%            u1, u2, u3, u4 vectors in class 2 then input Data is to be
%            formed as
%
%            Data = [v1; v2; v3; u1; u2; u3; u4],             (1)
%
%            where the size of vector v or vector u is 1xDim. Therefore the 
%            size of input Data in equation 1 would be 7xDim (depicting 
%            first 3 vectors belong to class 1 and second 4 vectors belong
%            to class 2). In the case of equation 1, class label is to be 
%            given as
%
%            Class = [1;1;1;2;2;2;2].                         (2)
%
%            size of Class (in equation 2) is 7x1.
%         
%   Output:                                                            
%          w: transformation matrix                                  
%    CPU_TIME : computational time                                 
%
%   Reference:
%   Sharma, A. and Paliwal, K.K., Linear discriminant analysis for the 
%      small sample size problem: a review, Pattern Recognition Letters
%   (in press), 2012
%
% 
%   Alok Sharma
%   University of Tokyo
%   Email: aloks@ims.u-tokyo.ac.jp
%
%--------------------------------------------------------------------------
 
t0 = cputime;

k        = max(Class); % number of classes

%-------------------------------------------------------------------------
[Nvec,n] = size(Data);
cc    = sum(Data)/Nvec;

for i = 1:k 
	loc                  = find(Class==i);
	[num,o]              = size(loc);
    vec(i)=num;
end;

Ht = (Data - ones(Nvec,1)*cc)';

[E1,Dt] = eig(Ht'*Ht);
dt=diag(Dt);
[row,col]=sort(dt,'descend');
rt = rank(Dt);
idx=col(1:rt);
Dt = Dt(idx,idx);
E1 = E1(:,idx);

dt=diag(Dt);

dt2=sqrt(dt);
Dt=diag(dt2);
Hth = Dt*E1'; %Dt^(1/2)*E1';

% floating error correction
corc = 1e+5;
Hth = corc*Hth; 
Hth = round(Hth)/corc;

dt2=sqrt(1./dt); %Dt^(-1/2)
dt3=1./dt;       %Dt^(-1)

for j=1:k
   rg=sum(vec(1:j-1));
   Hbh(:,j)=(1/sqrt(vec(j)))*(sum(Hth(:,1+rg:vec(j)+rg)')');
end
rb=k-1;
R = Hbh*Hbh';
Dt=diag(dt3);
R = Dt*R; % Dt^-1*Hbh*Hbh';

[U,D]=eig(R);
d=diag(D);
[row,col]=sort(d,'descend');
idx=col(1:rb);
U=U(:,idx);

Dt=diag(dt2); % Dt^-1/2
w = Dt*U;
w = E1*w;
w = Ht*w;     % w = Q1*U; where Q1 = Ht*E1*Dt^(-1/2);

CPU_TIME = cputime - t0;
