function [w,CPU_TIME]  = NLDA(Data, Class)
%--------------------------------------------------------------------------
% PCA + Null linear discriminant analysis
%
%   [w,CPU_TIME]  = NLDA(Data, Class)
%
%   Input:                                                             
%     Data:  data matrix (Each row is a data point) (NVec x Dim)
%     Class: class label (class 1, ..., k) 
%
%            Note: All vectors of a class is to be arranged together, e.g.,
%            if there are 2 classes with v1, v2, v3 vectors in class 1 and
%            u1, u2, u3, u4 vectors in class 2 then input Data is to be
%            formed as
%
%            Data = [v1; v2; v3; u1; u2; u3; u4],             (1)
%
%            where the size of vector v or vector u is 1xDim. Therefore the 
%            size of input Data in equation 1 would be 7xDim (depicting 
%            first 3 vectors belong to class 1 and second 4 vectors belong
%            to class 2). In the case of equation 1, class label is to be 
%            given as
%
%            Class = [1;1;1;2;2;2;2].                         (2)
%
%            size of Class (in equation 2) is 7x1.
%         
%   Output:                                                            
%          w: transformation matrix                                  
%    CPU_TIME : computational time                                 
%
%   Reference:
%   Sharma, A. and Paliwal, K.K., Linear discriminant analysis for the 
%      small sample size problem: a review, Pattern Recognition Letters
%   (in press), 2012
%
% 
%   Alok Sharma
%   University of Tokyo
%   Email: aloks@ims.u-tokyo.ac.jp
%
%--------------------------------------------------------------------------
 
t0 = cputime;
class       = max(Class); % number of classes

%-------------------------------------------------------------------------
[m,n] = size(Data);
cc    = sum(Data)/m;
for i = 1:class
	loc   = find(Class==i);
	[num,col] = size(loc);
    nvec(i)=num;
end
mu = mean(Data);
Hb=[]; Hw=[];
for i=1:class
    rg = 1+sum(nvec(1:i-1)):sum(nvec(1:i));
    x = Data(rg,:);
    mui = mean(x);
    Hb = [Hb, sqrt(nvec(i))*(mui-mu)'];
    Hw = [Hw, (x - ones(nvec(i),1)*mui)'];
end

Ht = (Data - ones(m,1)*cc)';

[Ut,Dt] = svd(Ht,0);
rt=rank(Dt);
Ut=Ut(:,1:rt);
Hwh=Ut'*Hw;
Hbh=Ut'*Hb;
Sbh=Hbh*Hbh';
[Uw,Dw]=eig(Hwh*Hwh');
rw=rank(Dw);
Uw=Uw(:,1:rt-rw);
Sbh2 = Uw'*Sbh*Uw;
[Ub,Db]=eig(Sbh2);
db = diag(Db);
rbh=rank(Db);
[Eval,inx] = sort(db,'descend');
Ub = Ub(:,inx(1:rbh));
w = Ut*Uw*Ub;

CPU_TIME = cputime - t0;
