function [w,TIME] = PCAplusLDA(Data,Class)
%--------------------------------------------------------------------------
% PCAplusLDA (or Fisherface): Principal component analysis + linear
%                             discriminant analysis
%
%   [w,CPU_TIME]  = PCAplusLDA(Data, Class)
%
%   Input:                                                             
%     Data:  data matrix (Each row is a data point) (NVec x Dim)
%     Class: class label (class 1, ..., k) 
%
%            Note: All vectors of a class is to be arranged together, e.g.,
%            if there are 2 classes with v1, v2, v3 vectors in class 1 and
%            u1, u2, u3, u4 vectors in class 2 then input Data is to be
%            formed as
%
%            Data = [v1; v2; v3; u1; u2; u3; u4],             (1)
%
%            where the size of vector v or vector u is 1xDim. Therefore the 
%            size of input Data in equation 1 would be 7xDim (depicting 
%            first 3 vectors belong to class 1 and second 4 vectors belong
%            to class 2). In the case of equation 1, class label is to be 
%            given as
%
%            Class = [1;1;1;2;2;2;2].                         (2)
%
%            size of Class (in equation 2) is 7x1.
%         
%   Output:                                                            
%          w: transformation matrix                                  
%    CPU_TIME : computational time                                 
%
%   Reference:
%   Sharma, A. and Paliwal, K.K., Linear discriminant analysis for the 
%      small sample size problem: a review, Pattern Recognition Letters
%   (in press), 2012
%
% 
%   Alok Sharma
%   University of Tokyo
%   Email: aloks@ims.u-tokyo.ac.jp
%
%--------------------------------------------------------------------------

TIME = cputime;

class=max(Class);
for i = 1:class
	loc  = find(Class==i);
	[num,col] = size(loc);
    nvec(i)=num;
end
mu = mean(Data);
Ht = [Data - ones(sum(nvec),1)*mu]';
rt = rank(Ht);

%#################### PCA reduction to N-c  ##########################
[Ut,Dt,Vt]=svd(Ht,0);
DimTo = sum(nvec)-class;
Ut=Ut(:,1:DimTo);
m = Ut'*Data';
m=m';
dim = rt;
mu = mean(m);
Hb=[]; Hw=[];
for i=1:class
    rg = 1+sum(nvec(1:i-1)):sum(nvec(1:i));
    x = m(rg,:);
    mui = mean(x);
    Hb = [Hb, sqrt(nvec(i))*(mui-mu)'];
    Hw = [Hw, (x - ones(nvec(i),1)*mui)'];
end
rb = rank(Hb);
%###############################################

Sw = Hw*Hw';
Sb = Hb*Hb';
Q = (Sw^-1)*Sb;
[w,Del] = eig(Q);
del = diag(Del);
[row,inx] = sort(del,'descend');
h=rb;
w = w(:,inx(1:h));
w = Ut*w;

TIME = cputime - TIME;

end

