function [w,CPU_TIME]  = PILDA(Data, Class)
%--------------------------------------------------------------------------
% PILDA Pseudoinverse Linear Discriminant Analysis 
%
%   [w,CPU_TIME]  = PILDA(Data, Class)
%
%   Input:                                                             
%     Data:  data matrix (Each row is a data point) (NVec x Dim)
%     Class: class label (class 1, ..., k) 
%
%            Note: All vectors of a class is to be arranged together, e.g.,
%            if there are 2 classes with v1, v2, v3 vectors in class 1 and
%            u1, u2, u3, u4 vectors in class 2 then input Data is to be
%            formed as
%
%            Data = [v1; v2; v3; u1; u2; u3; u4],             (1)
%
%            where the size of vector v or vector u is 1xDim. Therefore the 
%            size of input Data in equation 1 would be 7xDim (depicting 
%            first 3 vectors belong to class 1 and second 4 vectors belong
%            to class 2). In the case of equation 1, class label is to be 
%            given as
%
%            Class = [1;1;1;2;2;2;2].                         (2)
%
%            size of Class (in equation 2) is 7x1.
%         
%   Output:                                                            
%          w: transformation matrix                                  
%    CPU_TIME : computational time                                 
%
%   Reference:
%   Sharma, A. and Paliwal, K.K., Linear discriminant analysis for the 
%      small sample size problem: a review, Pattern Recognition Letters
%   (in press), 2012
%
% 
%   Alok Sharma
%   University of Tokyo
%   Email: aloks@ims.u-tokyo.ac.jp
%
%--------------------------------------------------------------------------
 
t0 = cputime;

class = max(Class); % number of classes

%-------------------------------------------------------------------------
[Nvec,n] = size(Data);
cc    = sum(Data)/Nvec;

for i = 1:class
	loc   = find(Class==i);
	[num,col] = size(loc);
    nvec(i)=num;
end

Ht = (Data - ones(Nvec,1)*cc)';
[Ut,Dt] = svd(Ht,0);
rt=rank(Dt);
Ut = Ut(:,1:rt);
m = Ut'*Data';
m=m';
mu = mean(m);
Hb=[]; Hw=[];
for i=1:class
    rg = 1+sum(nvec(1:i-1)):sum(nvec(1:i));
    x = m(rg,:);
    mui = mean(x);
    Hb = [Hb, sqrt(nvec(i))*(mui-mu)'];
    Hw = [Hw, (x - ones(nvec(i),1)*mui)'];
end
rb = rank(Hb);

Sb = Hb*Hb';
[Uw,Dw]=svd(Hw,0);
rw=rank(Dw);
Dw = Dw(1:rw,1:rw);
Uw = Uw(:,1:rw);
Swh = Uw*(Dw^-2)*Uw';
[wh,Dh] = eig(Swh*Sb);
dh = diag(Dh);
[Eval,inx] = sort(dh,'descend');
w=wh(:,inx(1:rb));
w = Ut*w;

CPU_TIME = cputime - t0;
end
